import * as fs from 'node:fs';
import * as stream from 'node:stream';
import * as ssh2 from 'ssh2';

/**
 * Export native Node.js access, open, type and mode file constants for comfort.
 */
declare const constants: typeof fs.constants;
/**
 * Asynchronously changes the permissions of a file.
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the file.
 * @param mode - File mode to set.
 * @returns Promise that resolves when the operation is complete.
 */
declare function chmod(ssh: ssh2.Client | null, path: fs.PathLike, mode: number): Promise<void>;
/**
 * Asynchronously changes owner and group of a file.
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the file.
 * @param uid - User ID to set.
 * @param gid - Group ID to set.
 * @returns Promise that resolves when the operation is complete.
 * @throws Error if neither uid nor gid is provided.
 */
declare function chown(ssh: ssh2.Client | null, path: fs.PathLike, uid: number, gid: number): Promise<void>;
/**
 * Non exposed options type from `fs.createReadStream`.
 */
type FsReadStreamOptions = Parameters<typeof fs.createReadStream>[1];
/**
 * Creates a readable stream for a file.
 * @param ssh - SSH client or null for local operation.
 * @param source - Path to the source file.
 * @param options - Options for creating the read stream.
 * @returns Promise that resolves with a ReadStream.
 *
 * @example
 *
 * ```typescript
 * stream = await fs.createReadStream(sshOrNull, 'hello')
 * stream.pipe(fs.createWriteStream('test.in'))
 * ```
 */
declare function createReadStream<T extends ssh2.Client | null>(ssh: T, source: fs.PathLike, options?: T extends null ? FsReadStreamOptions : ssh2.ReadStreamOptions): Promise<T extends null ? fs.ReadStream : ssh2.ReadStream>;
/**
 * Rejected error by `createReadStream`.
 */
declare class WriteStreamError extends Error {
    code?: string | number;
    errno?: number;
    syscall?: string;
    path?: string;
}
/**
 * Non exposed options type from `fs.createReadStream`.
 */
type FsWriteStreamOptions = Parameters<typeof fs.createWriteStream>[1];
/**
 * Creates a writable stream for a file.
 *
 * In the original `fs` API, `createWriteStream` is directly return instead of being available on a promise completion. The reason is due to the internal nature where we need to create an SFTP instance asynchronously before returning the the writable stream..
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the destination file.
 * @param options - Options for creating the write stream.
 * @returns Promise that resolves with a WriteStream.
 *
 * @example
 *
 * ```javascript
 * stream = await fs.createWriteStream sshOrNull, 'test.out'
 * fs.createReadStream('test.in').pipe stream
 * ````
 */
declare function createWriteStream<T extends null | ssh2.Client>(ssh: T, path: fs.PathLike, options?: T extends null ? FsWriteStreamOptions : ssh2.WriteStreamOptions): Promise<T extends null ? fs.WriteStream : ssh2.WriteStream>;
/**
 * Tests whether a file or directory exists.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to test.
 * @returns Promise that resolves with a boolean indicating existence.
 */
declare function exists(ssh: ssh2.Client | null, path: fs.PathLike): Promise<boolean>;
/**
 * Sets the access and modification times of a file.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the file.
 * @param atime - Access time to set.
 * @param mtime - Modification time to set.
 * @returns Promise that resolves when the operation is complete.
 */
declare function futimes(ssh: ssh2.Client | null, path: fs.PathLike, atime: fs.TimeLike, mtime: fs.TimeLike): Promise<void>;
/**
 * Retrieves the fs.Stats object for a symbolic link.
 *
 * The function returns the fs.Stats object. lstat() is identical to stat(), except that if path is a symbolic link, then the link itself is stat-ed, not the file that it refers to.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the symbolic link.
 * @returns Promise that resolves with the fs.Stats or ssh2.Stats object.
 */
declare function lstat(ssh: ssh2.Client | null, path: fs.PathLike): Promise<fs.Stats | ssh2.Stats>;
/**
 * Options for `mkdir`.
 *
 * Options extend the Node.js native options with the additional `gid` and `uid` options.
 */
type FsMkdirOptions = fs.MakeDirectoryOptions & {
    recursive: true;
} & {
    gid?: number;
    uid?: number;
};
/**
 * Creates a directory.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path of the directory to create.
 * @param options - Options for directory creation.
 * @returns Promise that resolves when the operation is complete.
 */
declare function mkdir(ssh: ssh2.Client | null, path: fs.PathLike, options?: fs.Mode | FsMkdirOptions | ssh2.InputAttributes): Promise<void>;
/**
 * Reads the contents of a directory.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the directory.
 * @returns Promise that resolves with an array of file names in the directory.
 */
declare function readdir(ssh: ssh2.Client | null, path: fs.PathLike): Promise<string[]>;
/**
 * Non exposed options type from `fs.promises.readFile`.
 */
type FsReadFileOptions = Parameters<typeof fs.promises.readFile>[1];
/**
 * Reads the entire contents of a file.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the file.
 * @param options - Options for reading the file.
 * @params options.encoding - Encoding used to convert a buffer into a string.
 * @params options.flag - File system flag, default is `r`.
 * @returns Promise that resolves with the file contents as a string or Buffer.
 */
declare function readFile<T extends ssh2.Client | null>(ssh: T, path: fs.PathLike, options?: T extends null ? FsReadFileOptions : BufferEncoding | ssh2.ReadFileOptions): Promise<string | Buffer>;
/**
 * Reads the value of a symbolic link.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the symbolic link.
 * @returns Promise that resolves with the link's string value.
 */
declare function readlink(ssh: ssh2.Client | null, path: fs.PathLike): Promise<string>;
/**
 * Renames a file or directory.
 *
 * @param ssh - SSH client or null for local operation.
 * @param source - Current path of the file or directory.
 * @param target - New path for the file or directory.
 * @returns Promise that resolves when the operation is complete.
 */
declare function rename(ssh: ssh2.Client | null, source: fs.PathLike, target: fs.PathLike): Promise<void>;
/**
 * Removes a directory.
 *
 * @param ssh - SSH client or null for local operation.
 * @param target - Path of the directory to remove.
 * @returns Promise that resolves when the operation is complete.
 */
declare function rmdir(ssh: ssh2.Client | null, target: fs.PathLike): Promise<void>;
/**
 * Retrieves the Stats object for a file or directory.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path to the file or directory.
 * @returns Promise that resolves with the fs.Stats or ssh2.Stats object.
 */
declare function stat<T extends null | ssh2.Client>(ssh: T, path: fs.PathLike): Promise<T extends null ? fs.Stats : ssh2.Stats>;
/**
 * Creates a symbolic link.
 *
 * @param ssh - SSH client or null for local operation.
 * @param srcpath - Path that the symlink should point to.
 * @param dstpath - Path where the symbolic link should be created.
 * @returns Promise that resolves when the operation is complete.
 */
declare function symlink(ssh: ssh2.Client | null, srcpath: fs.PathLike, dstpath: fs.PathLike): Promise<void>;
/**
 * Removes a file.
 *
 * @param ssh - SSH client or null for local operation.
 * @param path - Path of the file to remove.
 * @returns Promise that resolves when the operation is complete.
 */
declare function unlink(ssh: ssh2.Client | null, path: fs.PathLike): Promise<void>;
/**
 * Extends the WriteFileOptions with `uid` and `gid` options.
 */
type WriteFileOptions<T> = (T extends null ? FsWriteStreamOptions : ssh2.WriteStreamOptions) & {
    uid?: number;
    gid?: number;
};
/**
 * Asynchronously writes data to a file, replacing the file if it already exists.
 *
 * @param ssh - SSH client or null for local operation.
 * @param target - Path of the file to write.
 * @param data - The data to write to the file.
 * @param options - Options for writing the file.
 * @param options - Buffer encoding, default to `utf8`.
 * @param options.gid - Unix group name or id who owns the target file, not in the original Node.js implementation..
 * @param options.flag - File system flag.
 * @param options.uid - Unix user name or id who owns the target file, not in the original Node.js implementation.
 * @returns Promise that resolves when the operation is complete.
 */
declare function writeFile<T extends null | ssh2.Client>(ssh: T, target: fs.PathLike, data: stream.Readable | Buffer | string, options?: WriteFileOptions<T>): Promise<void>;
declare const _default: {
    constants: typeof fs.constants;
    chmod: typeof chmod;
    chown: typeof chown;
    createReadStream: typeof createReadStream;
    WriteStreamError: typeof WriteStreamError;
    createWriteStream: typeof createWriteStream;
    exists: typeof exists;
    futimes: typeof futimes;
    lstat: typeof lstat;
    mkdir: typeof mkdir;
    readdir: typeof readdir;
    readFile: typeof readFile;
    readlink: typeof readlink;
    rename: typeof rename;
    rmdir: typeof rmdir;
    stat: typeof stat;
    symlink: typeof symlink;
    unlink: typeof unlink;
    writeFile: typeof writeFile;
};

export { type FsMkdirOptions, type FsReadFileOptions, type FsReadStreamOptions, type FsWriteStreamOptions, type WriteFileOptions, WriteStreamError, chmod, chown, constants, createReadStream, createWriteStream, _default as default, exists, futimes, lstat, mkdir, readFile, readdir, readlink, rename, rmdir, stat, symlink, unlink, writeFile };
